/******************************************************************************/
/*                                  _ASM.H                                    */
/*                                                                            */
/*          Macro definitions for TopSpeed C/C++ inline assembly              */
/*                                                                            */
/*  Sean Wilson, 22/May/94                                                    */
/******************************************************************************/
#ifndef _ASM_H
#define _ASM_H

/******************************************************************************/
/* The following macros constitute the inline 'assembler'.  The notation is   */
/* derived from that used in the Intel data sheets.  The macros whose names   */
/* begin with an underscore '_' are used to construct the operand codes and   */
/* should not be used in program code.  It is not possible to detect          */
/* programming errors in the use of these macros so some care must be taken.  */
/* Instructions and operands are carefully named to avoid any confusion       */
/* regarding the types of parameters.                                         */
/*                                                                            */
/* The 'assembler instructions' use the common names augmented to give        */
/* information about the types of operands that may be used:                  */
/*                                                                            */
/*  r:          Register operand                                              */
/*  s:          Segment register operand                                      */
/*  m:          Base register for memory operand                              */
/*  b:          8-bit constant                                                */
/*  w:          16-bit constant                                               */
/*  d:          Constant displacement/offset                                  */
/*                                                                            */
/* Naming conventions used for operands are:                                  */
/*                                                                            */
/*  r8:         8-bit register, e.g: AL, BH                                   */
/*  r16:        16-bit register, e.g: AX, SI                                  */
/*  r,r1,r2:    8 or 16-bit registers, e.g: AH, BX                            */
/*  s:          Segment register, e.g: DS, ES                                 */
/*  m:          BX,BP,SI or DI base register for memory operand               */
/*  c:          Either CL or 1                                                */
/*  b:          8-bit quantity or byte                                        */
/*  w:          16-bit quantity or word                                       */
/*                                                                            */
/* NOTES:                                                                     */
/*  Some instructions have multiple variants to handle different forms or     */
/*  memory opernd widths that cannot be generalised with the preprocessor.    */
/*                                                                            */
/*  Register values are encoded using the values necessary for the reg field  */
/*  of the instructions 'mod' byte with the addition of the width bit.  This  */
/*  simplifies the instruction macros.                                        */
/*                                                                            */
/*  In general, only 8-bit displacements for memory operands are supported.   */
/******************************************************************************/

/*** Byte registers                                                         ***/
#define AL      0x00
#define CL      0x08
#define DL      0x10
#define BL      0x18
#define AH      0x20
#define CH      0x28
#define DH      0x30
#define BH      0x38

/*** Word register field value, including width bit                         ***/
#define AX      0x01
#define CX      0x09
#define DX      0x11
#define BX      0x19
#define SP      0x21
#define BP      0x29
#define SI      0x31
#define DI      0x39

/*** Segment registers                                                      ***/
#define ES      0x00
#define CS      0x08
#define SS      0x10
#define DS      0x18
#define FS      0x20
#define GS      0x28

/*** Operand encoding                                                       ***/
#define _i8(b)          ((b)&0xFF)          /* Byte value                     */
#define _i16(w)         _i8(w),_i8((w)>>8)  /* Byte switched (intel) word     */

#define _mr             0x03                /* _r_m() is a reg field          */
#define _m0             0x00                /* no displacement                */
#define _m8             0x01                /* 8-bit displacement             */
#define _m16            0x02                /* 16-bit displacement            */
#define _mod(m)         ((m<<6)&0xC0)       /* Select mod field value         */
#define _reg(r)         (r&0x38)            /* Select reg field value         */
#define _r_m(r)         ((r>>3)&0x07)       /* Select r/m field value         */
#define _mrm(m,r,rm)    (_mod(m)|_reg(r)|_r_m(rm))
#define _s(v)           ((v<0)<<1)          /* Sign bit                       */
#define _w(v)           (v&1)               /* Width bit                      */

/*** Helper macros for indirect addressing.  The _at(r) macro takes the     ***/
/*** register values BX, BP, SI or DI as its parameters and constructs a    ***/
/*** r/m value for indirect addressing using the specified register         ***/
#define _ds             0x21                /* Default segment for _at()      */
#define _BX_BP_SI_DI    0x2CDB8L            /* Bitmap for base registers      */
#define _at(m)          (_BX_BP_SI_DI>>(((m-BX)>>3)*3))


/*** Helper macros for dealing with memory model differences                ***/
#if defined(_fdata)
 #define _ptr(o,s)      o,s                 /* Pointer in reg_param() pragma  */
 #define FARDATA(e)     e                   /* Far data contitional           */
 #define NEARDATA(e)                        /* Near data conditional          */
#else
 #define _ptr(o,s)      o                   /* Pointer in reg_param() pragma  */
 #define FARDATA(e)                         /* Far data conditional           */
 #define NEARDATA(e)    e                   /* Near data conditional          */
#endif

#if defined(_fcall)
 #define FARCALL(e)     e                   /* Far code conditional           */
 #define NEARCALL(e)                        /* Near code conditional          */
#else
 #define FARCALL(e)                         /* Far code conditional           */
 #define NEARCALL(e)    e                   /* Near code conditional          */
#endif

/*** Inline assembler commands                                              ***/

#define USE(s)          0x26|_reg(s)        /* Segment override prefix        */

#define AAA             0x37
#define AAD             0xD5,0x0A
#define AAM             0xD4,0x0A
#define AAS             0x3F
#define ADD(r,b)        0x80|_s(b)|_w(r),_mrm(_mr,0x00,r),_i8(b)
#define ADDw(r,w)       0x80|_w(r),_mrm(_mr,0x00,r),_i16(w)
#define ADC(r,b)        0x80|_s(b)|_w(r),_mrm(_mr,0x10,r),_i8(b)
#define ADCw(r,w)       0x80|_w(r),_mrm(_mr,0x10,r),_i16(w)
#define AND(r1,r2)      0x22|_w(r1),_mrm(_mr,r1,r2)
#define ANDb(r,b)       0x80,_mrm(_mr,0x20,r),_i8(b)
#define ANDw(r,w)       0x81,_mrm(_mr,0x20,r),_i16(w)
#define CBW             0x98
#define CLC             0xF8
#define CLD             0xFC
#define CLI             0xFA
#define CMC             0xF5
#define CMPSb           0xA6
#define CMPSw           0xA7
#define CMP(r1,r2)      0x3A|_w(r1),_mrm(_mr,r1,r2)
#define CMPb(r,b)       0x80|_s(b),_mrm(_mr,0x38,r),_i8(b)
#define CMPw(r,w)       0x81|_s(w),_mrm(_mr,0x38,r),_i16(w)
#define CWD             0x99
#define DAA             0x27
#define DAS             0x2F
#define DEC(r)          0x48|_r_m(r)
#define DECb(m)         0xFE,_mrm(_m0,0x08,_at(m))
#define DECw(m)         0xFF,_mrm(_m0,0x08,_at(m))
#define DECbmd(m,b)     0xFE,_mrm(_m8,0x08,_at(m)),_i8(b)
#define DECwmd(m,b)     0xFF,_mrm(_m8,0x08,_at(m)),_i8(b)
#define DIV(r)          0xF6|_w(r),_mrm(_mr,0x30,r)
#define DIVb(m)         0xF6,_mrm(_m0,0x30,_at(m))
#define DIVw(m)         0xF7,_mrm(_m0,0x30,_at(m))
#define DIVbmd(m,b)     0xF6,_mrm(_m8,0x30,_at(m)),_i8(b)
#define DIVwmd(m,b)     0xF7,_mrm(_m8,0x30,_at(m)),_i8(b)
#define HLT             0xF4
#define IDIV(r)         0xF6|_w(r),_mrm(_mr,0x38,r)
#define IDIVb(m)        0xF6,_mrm(_m0,0x38,_at(m))
#define IDIVw(m)        0xF7,_mrm(_m0,0x38,_at(m))
#define IDIVbmd(m,b)    0xF6,_mrm(_m8,0x38,_at(m)),_i8(b)
#define IDIVwmd(m,b)    0xF7,_mrm(_m8,0x38,_at(m)),_i8(b)
#define INC(r)          0x40|_r_m(r)
#define INCb(m)         0xFE,_mrm(_m0,0x00,_at(m))
#define INCw(m)         0xFF,_mrm(_m0,0x00,_at(m))
#define INCbmd(m,b)     0xFE,_mrm(_m8,0x00,_at(m)),_i8(b)
#define INCwmd(m,b)     0xFF,_mrm(_m8,0x00,_at(m)),_i8(b)
#define INPb(b)         0xE4,_i8(b)
#define INPw(b)         0xE5,_i8(b)
#define INT(b)          0xCD,_i8(b)
#define INTO            0xCE
#define INT3            0xCC
#define JAE(b)          0x73,_i8(b)
#define JC(b)           0x72,_i8(b)
#define JCXZ(b)         0xE3,_i8(b)
#define JE(b)           0x74,_i8(b)
#define JNE(b)          0x75,_i8(b)
#define JMP(b)          0xEB,_i8(b)
#define JNC(b)          0x73,_i8(b)
#define LAHF            0x9D
#define LEA(r1,r2)      0x8D,_mrm(_mr,r1,r2)
#define LEArm(r,m,b)    0x8D,_mrm(_m8,r,_at(m)),_i8(b)
#define LEArd(r,w)      0x8D,_mrm(_m0,r,_at(_ds)),_i16(w)
#define LDS(r1,r2)      0xC5,_mrm(_mr,r1,r2)
#define LDSrm(r,m,b)    0xC5,_mrm(_m8,r,_at(m)),_i8(b)
#define LDSrd(r,w)      0xC5,_mrm(_m0,r,_at(_ds)),_i16(w)
#define LES(r1,r2)      0xC4,_mrm(_mr,r1,r2)
#define LESrm(r,m,b)    0xC4,_mrm(_m8,r,_at(m)),_i8(b)
#define LESrd(r,w)      0xC4,_mrm(_m0,r,_at(_ds)),_i16(w)
#define LOCK            0xF0
#define LODSb           0xAC
#define LODSw           0xAD
#define LOOP(b)         0xE2,_i8(b)
#define LOOPE(b)        0xE1,_i8(b)
#define LOOPNE(b)       0xE0,_i8(b)
#define LSL(r1,r2)      0x0F,0x03,_mrm(_mr,r1,r2)
#define MOV(r1,r2)      0x8A|_w(r1),_mrm(_mr,r1,r2)
#define MOVrb(r8,b)     0xB0|((_w(r8))<<3)|_r_m(r8),_i8(b)
#define MOVrw(r16,w)    0xB0|((_w(r16))<<3)|_r_m(r16),_i16(w)
#define MOVsr(s,r16)    0x8E,_mrm(_mr,s,r16)
#define MOVrs(r16,s)    0x8C,_mrm(_mr,s,r16)
#define MOVsm(s,m)      0x8E,_mrm(_m0,s,_at(m))
#define MOVms(m,s)      0x8C,_mrm(_m0,s,_at(m))
#define MOVrm(r,m)      0x8A|_w(r),_mrm(_m0,r,_at(m))
#define MOVmr(m,r)      0x88|_w(r),_mrm(_m0,r,_at(m))
#define MOVrd(r,w)      0x8A|_w(r),_mrm(_m0,r,_at(_ds)),_i16(w)
#define MOVdr(w,r)      0x88|_w(r),_mrm(_m0,r,_at(_ds)),_i16(w)
#define MOVrmd(r,m,b)   0x8A|_w(r),_mrm(_m8,r,_at(m)),_i8(b)
#define MOVmdr(m,b,r)   0x88|_w(r),_mrm(_m8,r,_at(m)),_i8(b)
#define MOVrmw(r,m,w)   0x8A|_w(r),_mrm(_m16,r,_at(m)),_i16(w)
#define MOVmwr(m,w,r)   0x88|_w(r),_mrm(_m16,r,_at(m)),_i16(w)
#define MOVSb           0xA4
#define MOVSw           0xA5
#define NEG(r)          0xF6|_w(r),_mrm(_mr,0x18,r)
#define NOP             0x90
#define NOT(r)          0xF6|_w(r),_mrm(_mr,0x10,r)
#define OR(r1,r2)       0x0A|_w(r1),_mrm(_mr,r1,r2)
#define ORb(r,b)        0x80|_s(b)|_w(r),_mrm(_mr,0x08,r),_i8(b)
#define ORw(r,w)        0x80|_w(r),_mrm(_mr,0x08,r),_i16(w)
#define OUTPb(b)        0xE6,_i8(b)
#define OUTPw(b)        0xE7,_i8(b)
#define POP(r)          0x58|_r_m(r)
#define POPs(s)         0x07|s
#define POPA            0x61
#define POPF            0x9D
#define PUSH(r)         0x50|_r_m(r)
#define PUSHs(s)        0x06|s
#define PUSHA           0x60
#define PUSHF           0x9C
#define REP             0xF2
#define REPZ            0xF3
#define RET_NEARN(w)    0xC2,_i16(w)
#define RET_NEAR        0xC3
#define RET_FARN(w)     0xCA,_i16(w)
#define RET_FAR         0xCB
#define RCL(r,c)        0xD0|((c==CL)<<1)|_w(r),_mrm(_mr,0x10,r)
#define RCR(r,c)        0xD0|((c==CL)<<1)|_w(r),_mrm(_mr,0x18,r)
#define ROL(r,c)        0xD0|((c==CL)<<1)|_w(r),_mrm(_mr,0x00,r)
#define ROR(r,c)        0xD0|((c==CL)<<1)|_w(r),_mrm(_mr,0x08,r)
#define SAL(r,c)        0xD0|((c==CL)<<1)|_w(r),_mrm(_mr,0x20,r)
#define SAR(r,c)        0xD0|((c==CL)<<1)|_w(r),_mrm(_mr,0x38,r)
#define SHL(r,c)        0xD0|((c==CL)<<1)|_w(r),_mrm(_mr,0x20,r)
#define SHR(r,c)        0xD0|((c==CL)<<1)|_w(r),_mrm(_mr,0x28,r)
#define SAHF            0x9E
#define SCASb           0xAE
#define SCASw           0xAF
#define STC             0xF9
#define STD             0xFD
#define STI             0xFB
#define STOSb           0xAA
#define STOSw           0xAB
#define SUB(r1,r2)      0x28|_w(r1),_mrm(_mr,r2,r1)
#define SUBb(r,b)       0x80|_s(b)|_w(r),_mrm(_mr,0x28,r),_i8(b)
#define SUBw(r,w)       0x80|_w(r),_mrm(_mr,0x28,r),_i16(w)
#define SBB(r1,r2)      0x18|_w(r1),_mrm(_mr,r2,r1)
#define SBBb(r,b)       0x80|_s(b)|_w(r),_mrm(_mr,0x18,r),_i8(b)
#define SBBw(r,w)       0x80|_w(r),_mrm(_mr,0x18,r),_i16(w)
#define TEST(r1,r2)     0x84|_w(r1),_mrm(_mr,r1,r2)
#define TESTb(r,b)      0xF6|_s(b)|_w(r),_mrm(_mr,0x00,r),_i8(b)
#define TESTw(r,w)      0xF6|_w(r),_mrm(_mr,0x00,r),_i16(b)
#define VERR(r)         0x0F,0x00,_mrm(_mr,0x20,r)
#define VERW(r)         0x0F,0x00,_mrm(_mr,0x28,r)
#define WAIT            0x9B
#define XCHG(r1,r2)     0x86|_w(r1),_mrm(_mr,r1,r2)
#define XCHGa(r)        0x90|r
#define XLAT            0xD7
#define XOR(r1,r2)      0x32|_w(r1),_mrm(_mr,r1,r2)

#undef RET
#if defined(_fcall)
 #define RET            RET_FAR
 #define RETN(w)        RET_FARN(w)
#else
 #define RET            RET_NEAR
 #define RETN(w)        RET_NEARN(w)
#endif

#endif /* _ASM_H */