/******************************************************************************/
/*                                  _DOS.H                                    */
/*                                                                            */
/*  This header defines structures used internally by DOS, many of which are  */
/*  undocumented.  Inline DOS functions are defined using inline assembler.   */
/*  They are intended to be simple to use and as efficient as possible.       */
/*  Where possible the return value of the function is an error-code with     */
/*  other values being returned in pointer parameters.  This ensures that     */
/*  there is no loss of information.  This approach is similar to that used   */
/*  in other PC Operating System API's.                                       */
/*                                                                            */
/*  Sean Wilson, 22/May/94                                                    */
/******************************************************************************/
#ifndef _DOS_H
#define _DOS_H

#include "_asm.h"

#if !defined(_fdata)  ||  !defined(_WIN)
#undef LNF_SUPPORT
#elif defined(LFN_SUPPORT)
#include "lfnsupp.h"
#endif


/*** DOS error codes                                                        ***/
#define _DOSEBADFNCODE      1               /* Invalid function code          */
#define _DOSENOENT          2               /* File not found                 */
#define _DOSENOPATH         3               /* Path not found                 */
#define _DOSENOHND          4               /* Too many open files            */
#define _DOSEACCESS         5               /* Access denied                  */
#define _DOSEBADHND         6               /* Invalid handle                 */
#define _DOSECORMEM         7               /* Arena trashed (memory corrupt) */
#define _DOSENOMEM          8               /* Insufficient memory            */
#define _DOSEBADBLK         9               /* Invalid memory block           */
#define _DOSEBADENV         10              /* Invalid environment            */
#define _DOSEBADFMT         11              /* Invalid format                 */
#define _DOSEBADACC         12              /* Invalid access code            */
#define _DOSEBADDAT         13              /* Invalid data                   */
#define _DOSEBADDRV         15              /* Invalid drive                  */
#define _DOSECURDIR         16              /* Attempt to remove current dir  */
#define _DOSEBADDEV         17              /* Not the same device            */
#define _DOSENOFILE         18              /* No more files                  */
#define _DOSEWPROT          19              /* Disk is write protected        */
#define _DOSEBADDSK         20              /* Bad disk unit                  */
#define _DOSENOTREADY       21              /* Drive not ready                */
#define _DOSEBADCMD         22              /* Unknown command                */
#define _DOSEBADCRC         23              /* CRC error                      */
#define _DOSEBADLEN         24              /* Bad request structure length   */
#define _DOSESEEK           25              /* Seek error                     */
#define _DOSBADMEDIA        26              /* Unknown media type             */
#define _DOSEBADSECT        27              /* Sector not found               */
#define _DOSENOPAPER        28              /* Printer out of paper           */
#define _DOSEWRFAULT        29              /* Write fault                    */
#define _DOSERDFAULT        30              /* Read fault                     */
#define _DOSEGENFAIL        31              /* General failure                */
#define _DOSESHRVIOL        32              /* Sharing violation              */
#define _DOSELCKVIOL        33              /* Lock violation                 */
#define _DOSEBADCHG         34              /* Invalid disk change            */
#define _DOSENOFCB          35              /* FCB unavailable                */
#define _DOSESHOFLO         36              /* Sharing buffer overflow        */
#define _DOSEFEXISTS        96              /* File exists                    */
#define _DOSENOFN           98              /* Cannot make function           */
#define _DOSEFAIL24         99              /* Fail on INT24                  */
#define _DOSENOSTRC         100             /* Out of structures              */
#define _DOSEASGND          101             /* Already assigned               */
#define _DOSBADPWD          102             /* Invalid password               */
#define _DOSBADPARM         103             /* Invalid parameter              */
#define _DOSENWRFAULT       104             /* Network write fault            */

#define _NETBADREQ          66              /* Network request not supported  */
#define _NETENOREM          67              /* Remote computer not listening  */


/*** Flags for DosFCBParseFilename()                                        ***/
#define _DOSFCBPRSLSEP      0x01            /* Scan off leading separators    */
#define _DOSFCBPRSDVID      0x02            /* Set drive ID byte              */
#define _DOSFCBPRSFLNM      0x04            /* Set filename                   */
#define _DOSFCBPRSEXT       0x08            /* Set extension                  */


/*** File attributes for _DosCreateFile()                                   ***/
#define _DOS_FANORMAL       0x00
#define _DOS_FAREADONLY     0x01
#define _DOS_FAHIDDEN       0x02
#define _DOS_FASYSTEM       0x03


/*** File attributes for _DosOpenFile()                                     ***/
#define _DOS_OAREADONLY     0x00
#define _DOS_OAWRITEONLY    0x01
#define _DOS_OAREADWRITE    0x02


/*** Share flags for _DosOpenFile()                                         ***/
#define _DOS_SHCOMPAT       0x00
#define _DOS_SHDENYRW       0x10
#define _DOS_SHDENYWR       0x20
#define _DOS_SHDENYRD       0x30
#define _DOS_SHDENYNO       0x40
#define _DOS_NOINHERIT      0x80


/*** Origin for _DosSeekFile() and _DosRepositionFile()                     ***/
#define _DOS_SEEKSET    0
#define _DOS_SEEKCUR    1
#define _DOS_SEEKEND    2


/*** Retur codes for _DosProcessReturnCode()                                ***/
#define _DOSRETNORMAL                       /* Normal termination             */
#define _DOSRETCTRLBRK                      /* Ctrl-break termination         */
#define _DOSRETCRIT                         /* Critical error termination     */
#define _DOSRETTSR                          /* Stayed TSR                     */


/*** DOS structures and functions                                           ***/
typedef unsigned char   byte;
typedef unsigned long   dword;


typedef struct _DOSPSP {                    /* DOS Program Segment Prefix     */
    unsigned    pspInt21;
    unsigned    pspNextPara;
    byte    _1;
    byte    pspDispatcher[5];
    dword   pspTerminateVec;
    dword   pspControlCVec;
    dword   pspCritErrVec;
    unsigned    _2[11];
    unsigned    pspEnv;
    unsigned    _3[23];
    byte    pspFCB_1[16];
    byte    pspFCB_2[16];
    dword   _4;
    byte    pspCommandTail[128];
}   _DOSPSP;


/* Memory control Block:  mcbTag contains contains 'M' except for the last    */
/* MCB in the chain in which the tag is 'Z'.  mcbOwnerId is the lo:hi PSP of  */
/* the owner, 0008 if the owner is DOS, or 0000 if the MCB is free.  mcbSize  */
/* contains the size in paragraphs of the memory block controlled by the MCB, */
/* not including the MCB structure.  In DOS 4.x the final 8 bytes contain the */
/* program name of the owner, if the MCB contains the PSP.                    */
typedef struct _DOSMCB {
    byte    mcbTag;
    unsigned    mcbOwnerId;
    unsigned    mcbSize;
    byte    _[3];
    byte    mcbOwnerName[8];
}   _DOSMCB;


typedef struct _DOSFCB {                    /* DOS File Control Block         */
    byte    fcbDrive;                       /* 0=Default, 1=A, etc.           */
    byte    fcbName[8];                     /* Name padded with blanks        */
    byte    fcbExt[3];                      /* Extension padded with blanks   */
    unsigned    fcbCurrentBlock;                /* Current block number           */
    unsigned    fcbRecordSize;                  /* Logical record size            */
    dword   fcbFileSize;                    /* File length in bytes           */
    unsigned    fcbDate;                        /* Date of last write             */
    unsigned    fcbTime;                        /* Time of last write             */
    byte    _[8];                           /*                                */
    dword   fcbRecordNumber;                /* Random access record number    */
}   _DOSFCB;


typedef struct _DOSDATE {                   /* DOS Date Structure             */
    unsigned    dYear;                          /* Year (1980 - 2099)             */
    byte    dDay;                           /* Day (1 - 31)                   */
    byte    dMonth;                         /* Month (1 - 12)                 */
    byte    dDayOfWeek;                     /* Day of the week (0 = Sunday)   */
}   _DOSDATE;


typedef struct _DOSTIME {                   /* DOS Time Structure             */
    byte    tMinutes;                       /* Minutes (0 - 59)               */
    byte    tHour;                          /* Hour (0 - 23)                  */
    byte    tHundredths;                    /* Hundredths (0 - 99)            */
    byte    tSeconds;                       /* Seconds (0 - 59)               */
}   _DOSTIME;


typedef struct _DOSERROR {                  /* DOS Extended Error Structure   */
    unsigned    errCode;                        /* Error Code                     */
    byte    errAction;                      /* Error Action                   */
    byte    errClass;                       /* Error Class                    */
    byte    errLocus;                       /* Error Locus                    */
}   _DOSERROR;


typedef struct _DOSFATINFO {                /* DOS FAT Information Structure  */
    byte far *fatId;                        /* Pointer to FAT ID byte         */
    unsigned    fatClusters;                    /* Number of Clusters             */
    byte    fatSecPerClus;                  /* Sectors per Cluster            */
    unsigned    fatBytePerSec;                  /* Bytes per sector               */
}   _DOSFATINFO;


typedef struct _DOSDISKINFO {               /* DOS Disk Information Structure */
    unsigned    dskFreeClusters;                /* Available Clusters             */
    unsigned    dskTotalClusters;               /* Total Clusters                 */
    unsigned    dskBytesPerSector;              /* Bytes per sector               */
    unsigned    dskSectorsPerCluster;           /* Sectors per Cluster            */
}   _DOSDISKINFO;


#pragma save, call(inline=>on, reg_param=>(bx), reg_saved=>(bx,cx,dx,si,di,ds,es,st1,st2))
static void _DosSetPSP(unsigned PSPSeg) =
{   MOVrb(AH,0x50),
    INT(0x21)
};
#pragma restore


#pragma save, call(inline=>on, reg_return=>(bx), reg_saved=>(cx,dx,si,di,ds,es,st1,st2))
static unsigned _DosGetPSP(void) =
{   MOVrb(AH,0x51),
    INT(0x21)
};
#pragma restore


#pragma save, call(inline=>on, reg_param=>(ax), reg_saved=>(ax,bx,cx,dx,si,di,ds,es,st1,st2))
static void _DosTerminateProgram(unsigned PSPSeg) =
{   MOVsr(CS,AX),
    MOVrb(AH,0x00),
    INT(0x21)
};
#pragma restore


#pragma save, call(inline=>on, reg_param=>(), reg_saved=>(bx,cx,dx,si,di,ds,es,st1,st2))
static byte _DosReadKbdWithEcho(void) =
{   MOVrb(AH,0x01),
    INT(0x21)
};
#pragma restore


#pragma save, call(inline=>on, reg_param=>(dx), reg_saved=>(ax,bx,cx,dx,si,di,ds,es,st1,st2))
static void _DosDisplayCharacter(byte C) =
{   MOVrb(AH,0x02),
    INT(0x21)
};
#pragma restore


#pragma save, call(inline=>on, reg_param=>(), reg_saved=>(bx,cx,dx,si,di,ds,es,st1,st2))
static byte _DosAuxInput(void) =
{   MOVrb(AH,0x03),
    INT(0x21)
};
#pragma restore


#pragma save, call(inline=>on, reg_param=>(dx), reg_saved=>(ax,bx,cx,dx,si,di,ds,es,st1,st2))
static void _DosAuxOutput(byte C) =
{   MOVrb(AH,0x04),
    INT(0x21)
};
#pragma restore


#pragma save, call(inline=>on, reg_param=>(dx), reg_saved=>(ax,bx,cx,dx,si,di,ds,es,st1,st2))
static void _DosPrintCharacter(byte C) =
{   MOVrb(AH,0x05),
    INT(0x21)
};
#pragma restore


#pragma save, call(inline=>on, reg_param=>(dx), reg_saved=>(bx,cx,dx,si,di,ds,es,st1,st2))
static byte _DosConsole(byte C) =
{   MOVrb(AH,0x06),
    INT(0x21)
};
#pragma restore


#pragma save, call(inline=>on, reg_param=>(), reg_saved=>(bx,cx,dx,si,di,ds,es,st1,st2))
static byte _DosConsoleInput(void) =
{   MOVrb(AH,0x07),
    INT(0x21)
};
#pragma restore


#pragma save, call(inline=>on, reg_param=>(), reg_saved=>(bx,cx,dx,si,di,ds,es,st1,st2))
static byte _DosReadKbd(void) =
{   MOVrb(AH,0x08),
    INT(0x21)
};
#pragma restore


#pragma save, call(inline=>on, reg_param=>(dx,ds), reg_saved=>(ax,bx,cx,dx,si,di,ds,es,st1,st2))
static void _DosDisplayString(byte *String$) =
{   MOVrb(AH,0x09),
    INT(0x21)
};
#pragma restore


#pragma save, call(inline=>on, reg_param=>(dx,ds,ax), reg_saved=>(ax,bx,cx,dx,si,di,ds,es,st1,st2))
static void _DosBufferedInput(byte *Buf, byte Len) =
{   MOVrb(AH,0x0A),
    INT(0x21)
};
#pragma restore


#pragma save, call(inline=>on, reg_param=>(), reg_saved=>(bx,cx,dx,si,di,ds,es,st1,st2))
static byte _DosKeyInBuffer(void) =
{   MOVrb(AH,0x0B),
    INT(0x21)
};
#pragma restore


#pragma save, call(inline=>on, reg_param=>(), reg_saved=>(ax,bx,cx,dx,si,di,ds,es,st1,st2))
static void _DosResetDrive(void) =
{   MOVrb(AH,0x0D),
    INT(0x21)
};
#pragma restore


#pragma save, call(inline=>on, reg_param=>(dx), reg_saved=>(bx,cx,dx,si,di,ds,es,st1,st2))
static byte _DosSetDefaultDrive(byte Drive) =
{   MOVrb(AH,0x0E),
    INT(0x21)
};
#pragma restore


#pragma save, call(inline=>on, reg_param=>(dx,ds), reg_saved=>(bx,cx,dx,si,di,ds,es,st1,st2))
static byte _DosFCBOpen(_DOSFCB *Fcb) =
{   MOVrb(AH,0x0F),
    INT(0x21)
};
#pragma restore


#pragma save, call(inline=>on, reg_param=>(dx,ds), reg_saved=>(bx,cx,dx,si,di,ds,es,st1,st2))
static byte _DosFCBClose(_DOSFCB *Fcb) =
{   MOVrb(AH,0x10),
    INT(0x21)
};
#pragma restore


#pragma save, call(inline=>on, reg_param=>(dx,ds), reg_saved=>(bx,cx,dx,si,di,ds,es,st1,st2))
static byte _DosFCBFindFirst(_DOSFCB *Fcb) =
{   MOVrb(AH,0x11),
    INT(0x21)
};
#pragma restore


#pragma save, call(inline=>on, reg_param=>(dx,ds), reg_saved=>(bx,cx,dx,si,di,ds,es,st1,st2))
static byte _DosFCBFindNext(_DOSFCB *Fcb) =
{   MOVrb(AH,0x12),
    INT(0x21)
};
#pragma restore


#pragma save, call(inline=>on, reg_param=>(dx,ds), reg_saved=>(bx,cx,dx,si,di,ds,es,st1,st2))
static byte _DosFCBDelete(_DOSFCB *Fcb) =
{   MOVrb(AH,0x13),
    INT(0x21)
};
#pragma restore


#pragma save, call(inline=>on, reg_param=>(dx,ds), reg_saved=>(bx,cx,dx,si,di,ds,es,st1,st2))
static byte _DosFCBRead(_DOSFCB *Fcb) =
{   MOVrb(AH,0x14),
    INT(0x21)
};
#pragma restore


#pragma save, call(inline=>on, reg_param=>(dx,ds), reg_saved=>(bx,cx,dx,si,di,ds,es,st1,st2))
static byte _DosFCBWrite(_DOSFCB *Fcb) =
{   MOVrb(AH,0x15),
    INT(0x21)
};
#pragma restore


#pragma save, call(inline=>on, reg_param=>(dx,ds), reg_saved=>(bx,cx,dx,si,di,ds,es,st1,st2))
static byte _DosFCBCreate(_DOSFCB *Fcb) =
{   MOVrb(AH,0x16),
    INT(0x21)
};
#pragma restore


#pragma save, call(inline=>on, reg_param=>(dx,ds), reg_saved=>(bx,cx,dx,si,di,ds,es,st1,st2))
static byte _DosFCBRename(_DOSFCB *Fcb) =
{   MOVrb(AH,0x17),
    INT(0x21)
};
#pragma restore


#pragma save, call(inline=>on, reg_param=>(), reg_saved=>(bx,cx,dx,si,di,ds,es,st1,st2))
static byte _DosGetCurrentDrive(void) =
{   MOVrb(AH,0x19),
    INT(0x21)
};
#pragma restore


#pragma save, call(inline=>on, reg_param=>(dx,ds), reg_saved=>(ax,bx,cx,dx,si,di,ds,es,st1,st2))
static void _DosSetDTA(void far *DTA) =
{   MOVrb(AH,0x1A),
    INT(0x21)
};
#pragma restore


#pragma save, call(inline=>on, reg_param=>(si,ds), reg_saved=>(si,ds,es,st1,st2))
static void _DosGetFatInfo(_DOSFATINFO *pInfo) =
{   MOVrb(AH,0x1B),
    PUSHs(DS),
    INT(0x21),
    MOVsr(DI,DS),
    POPs(DS),
    MOVrm(SI,BX),
    MOVrmd(SI,2,SI),
    MOVrmd(SI,4,DX),
    MOVrmd(SI,6,AL),
    MOVrmd(SI,7,CX),
};
#pragma restore


#pragma save, call(inline=>on, reg_param=>(), reg_return=>(bx,ds), reg_saved=>(ax,bx,cx,dx,si,di,ds,es,st1,st2))
//static _DOSBPB *_DosGetDPB() =
//{   MOVrb(AH,0x1F),
//    INT(0x21)
//};
#pragma restore


#pragma save, call(inline=>on, reg_param=>(dx,ds), reg_saved=>(bx,cx,dx,si,di,ds,es,st1,st2))
static byte _DosFCBRandomRead(_DOSFCB *Fcb) =
{   MOVrb(AH,0x21),
    INT(0x21)
};
#pragma restore


#pragma save, call(inline=>on, reg_param=>(dx,ds), reg_saved=>(bx,cx,dx,si,di,ds,es,st1,st2))
static byte _DosFCBRandomWrite(_DOSFCB *Fcb) =
{   MOVrb(AH,0x22),
    INT(0x21)
};
#pragma restore


#pragma save, call(inline=>on, reg_param=>(dx,ds), reg_saved=>(bx,cx,dx,si,di,ds,es,st1,st2))
static byte _DosFCBGetLength(_DOSFCB *Fcb) =
{   MOVrb(AH,0x23),
    INT(0x21)
};
#pragma restore


#pragma save, call(inline=>on, reg_param=>(dx,ds), reg_saved=>(bx,cx,dx,si,di,ds,es,st1,st2))
static void _DosFCBSetRelativeRecord(_DOSFCB *Fcb) =
{   MOVrb(AH,0x24),
    INT(0x21)
};
#pragma restore


#pragma save, call(inline=>on, reg_param=>(ax,dx,ds), reg_saved=>(bx,cx,dx,si,di,ds,es,st1,st2))
static void _DosSetInterruptVector(unsigned Int, void far *Vect) =
{   MOVrb(AH,0x25),
    INT(0x21)
};
#pragma restore


#pragma save, call(inline=>on, reg_param=>(dx), reg_saved=>(bx,cx,dx,si,di,ds,es,st1,st2))
static void _DosCreatePSP(unsigned PSP) =
{   MOVrb(AH,0x26),
    INT(0x21)
};
#pragma restore


#pragma save, call(inline=>on, reg_param=>(cx,dx,ds), reg_saved=>(bx,cx,dx,si,di,ds,es,st1,st2))
static byte _DosFCBRandomBlockRead(unsigned Cnt, _DOSFCB *Fcb) =
{   MOVrb(AH,0x27),
    INT(0x21)
};
#pragma restore


#pragma save, call(inline=>on, reg_param=>(cx,dx,ds), reg_saved=>(bx,cx,dx,si,di,ds,es,st1,st2))
static byte _DosFCBRandomBlockWrite(unsigned Cnt, _DOSFCB *Fcb) =
{   MOVrb(AH,0x28),
    INT(0x21)
};
#pragma restore


#pragma save, call(inline=>on, reg_param=>(ax,si,ds,di,es), reg_saved=>(bx,cx,dx,di,ds,es,st1,st2))
static byte _DosFCBParseFilename(unsigned Mode, char *CmdLine, _DOSFCB *Fcb) =
{   MOVrb(AH,0x29),
    INT(0x21),
};
#pragma restore


#pragma save, call(inline=>on, reg_param=>(si,ds), reg_saved=>(bx,si,di,ds,es,st1,st2))
static void _DosGetDate(_DOSDATE *pDate) =
{   MOVrb(AH,0x2A),
    INT(0x21),
    MOVmr(SI,CX),
    MOVmdr(SI,2,DX),
    MOVmdr(SI,4,AL)
};
#pragma restore


#pragma save, call(inline=>on, reg_param=>(si,ds), reg_saved=>(bx,si,di,ds,es,st1,st2))
static int _DosSetDate(_DOSDATE *pDate) =
{   MOVrb(AH,0x2B),
    MOVrm(CX,SI),
    MOVrmd(DX,SI,2),
    INT(0x21),
    JC(0x02),
    XOR(AX,AX),
};
#pragma restore


#pragma save, call(inline=>on, reg_param=>(si,ds), reg_saved=>(bx,si,di,ds,es,st1,st2))
static void _DosGetTime(_DOSTIME *pTime) =
{   MOVrb(AH,0x2C),
    INT(0x21),
    MOVmr(SI,CX),
    MOVmdr(SI,2,DX),
};
#pragma restore


#pragma save, call(inline=>on, reg_param=>(si,ds), reg_saved=>(bx,si,di,ds,es,st1,st2))
static int _DosSetTime(_DOSTIME *pTime) =
{   MOVrb(AH,0x2D),
    MOVrm(CX,SI),
    MOVrmd(DX,SI,2),
    INT(0x21),
    JC(0x02),
    XOR(AX,AX),
};
#pragma restore


#pragma save, call(inline=>on, reg_param=>(ax), reg_saved=>(bx,cx,dx,si,di,ds,es,st1,st2))
static void _DosSetVerify(unsigned Mode) =
{   MOVrb(AH,0x2E),
    INT(0x21)
};
#pragma restore


#pragma save, call(inline=>on, reg_param=>(), reg_return=>(bx,es), reg_saved=>(cx,dx,si,di,ds,es,st1,st2))
static void far *_DosGetDTA() =
{   MOVrb(AH,0x2F),
    INT(0x21)
};
#pragma restore


#pragma save, call(inline=>on, reg_param=>(ax), reg_saved=>(bx,si,di,ds,es,st1,st2))
static int _DosGetVersion() =
{   MOVrb(AH,0x30),
    INT(0x21)
};
#pragma restore


#pragma save, call(inline=>on, reg_param=>(ax,dx), reg_saved=>(bx,cx,dx,si,di,ds,es,st1,st2))
static void _DosTSR(int Ret, unsigned Paras) =
{   MOVrb(AH,0x31),
    INT(0x21)
};
#pragma restore


#pragma save, call(inline=>on, reg_param=>(ax), reg_return=>(bx,es), reg_saved=>(cx,dx,si,di,ds,es,st1,st2))
static void far *_DosGetInterruptVector(unsigned Int) =
{   MOVrb(AH,0x35),
    INT(0x21)
};
#pragma restore


#pragma save, call(inline=>on, reg_param=>(dx,si,ds), reg_saved=>(si,di,ds,es,st1,st2))
static int _DosGetDiskInfo(int Drv, _DOSDISKINFO *pInfo) =
{   MOVrb(AH,0x36),
    INT(0x21),
    CMPw(AX,0xFFFF),
    JNE(0x05),
    MOVrw(AX,_DOSEBADDRV),
    JMP(0x0B),
    MOVmr(SI,BX),
    MOVmdr(SI,2,DX),
    MOVmdr(SI,4,CX),
    MOVmdr(SI,6,AX),
};
#pragma restore


#pragma save, call(inline=>on, reg_param=>(dx,ds), reg_saved=>(bx,cx,dx,si,di,ds,es,st1,st2))
static int _SfnMakeDirectory(const char *Path) =
{
    MOVrb(AH,0x39),
    INT(0x21),
    JC(0x02),
    XOR(AX,AX)
};
#pragma restore

#pragma save, call(inline=>on, reg_param=>(dx,ds), reg_saved=>(bx,cx,dx,si,di,ds,es,st1,st2))
static int _SfnRemoveDirectory(const char *Path) =
{
    MOVrb(AH,0x3A),
    INT(0x21),
    JC(0x02),
    XOR(AX,AX)
};
#pragma restore


#pragma save, call(inline=>on, reg_param=>(dx,ds), reg_saved=>(bx,cx,dx,si,di,ds,es,st1,st2))
static int _SfnChangeDirectory(const char *Path) =
{
    MOVrb(AH,0x3B),
    INT(0x21),
    JC(0x02),
    XOR(AX,AX)
};
#pragma restore


#pragma save, call(inline=>on, reg_param=>(dx,ds,di,es,cx), reg_saved=>(bx,dx,si,di,ds,es,st1,st2))
static int _SfnCreateFile(const char *Path, int *pHnd, byte Attr) =
{
    MOVrb(CH,0),
    MOVrb(AH,0x3C),
    INT(0x21),
 #if _fdata
    JC(0x05),
    USE(ES), MOVmr(DI,AX),
 #else
    JC(0x04),
    MOVmr(DI,AX),
 #endif
    XOR(AX,AX)
};
#pragma restore


#pragma save, call(inline=>on, reg_param=>(dx,ds,di,es,ax), reg_saved=>(bx,cx,dx,si,di,ds,es,st1,st2))
static int _SfnOpenFile(const char *Path, int *pHnd, byte Access) =
{
    MOVrb(AH,0x3D),
    INT(0x21),
 #if _fdata
    JC(0x05),
    USE(ES), MOVmr(DI,AX),
 #else
    JC(0x04),
    MOVmr(DI,AX),
 #endif
    XOR(AX,AX),
};
#pragma restore

#pragma save, call(inline=>on, reg_param=>(dx,ds), reg_saved=>(bx,cx,dx,si,di,ds,es,st1,st2))
static int _SfnDeleteFile(const char *Path) =
{
    MOVrb(AH,0x41),
    INT(0x21),
    JC(0x02),
    XOR(AX,AX)
};
#pragma restore

#pragma save, call(inline=>on, reg_param=>(dx,ds,di,es), reg_saved=>(bx,dx,si,di,ds,es,st1,st2))
static int _SfnGetFileMode(const char *Path, int *pMode) =
{
    MOVrw(AX,0x4300),
    INT(0x21),
 #if _fdata
    JC(0x07),
    XOR(CH,CH),
    USE(ES), MOVmr(DI,CX),
 #else
    JC(0x06),
    XOR(CH,CH),
    MOVmr(DI,CX),
 #endif
    XOR(AX,AX)
};
#pragma restore


#pragma save, call(inline=>on, reg_param=>(dx,ds,cx), reg_saved=>(bx,dx,si,di,ds,es,st1,st2))
static int _SfnChgFileMode(const char *Path, int Mode) =
{
    MOVrw(AX,0x4301),
    XOR(CH,CH),
    INT(0x21),
    JC(0x02),
    XOR(AX,AX)
};
#pragma restore


#pragma save, call(inline=>on, reg_param=>(dx,si,ds), reg_saved=>(bx,cx,dx,si,di,ds,es,st1,st2))
static int _SfnGetCurrentDir(int Drv, char *pDir/*[64]*/) =
{
    MOVrb(AH,0x47),
    INT(0x21),
    JC(0x02),
    XOR(AX,AX)
};
#pragma restore


#pragma save, call(inline=>on, reg_param=>(dx,ds,di,es), reg_saved=>(bx,cx,dx,si,di,ds,es,st1,st2))
static int _SfnRenameFile(const char *OldPath, const char *NewPath) =
{
 #if !defined(_fdata)
    PUSHs(DS),
    POPs(ES),
 #endif
    MOVrb(AH,0x56),
    INT(0x21),
    JC(0x02),
    XOR(AX,AX)
};
#pragma restore



#ifdef LFN_SUPPORT

#define _DosMakeDirectory   _LfnMakeDirectory
#define _DosRemoveDirectory _LfnRemoveDirectory
#define _DosChangeDirectory _LfnChangeDirectory
#define _DosCreateFile      _LfnCreateFile
#define _DosOpenFile        _LfnOpenFile
#define _DosDeleteFile      _LfnDeleteFile
#define _DosGetFileMode     _LfnGetFileMode
#define _DosChgFileMode     _LfnChgFileMode
#define _DosGetCurrentDir   _LfnGetCurrentDir
#define _DosRenameFile      _LfnRenameFile

#else // no LFN_SUPPORT

#define _DosMakeDirectory   _SfnMakeDirectory
#define _DosRemoveDirectory _SfnRemoveDirectory
#define _DosChangeDirectory _SfnChangeDirectory
#define _DosCreateFile      _SfnCreateFile
#define _DosOpenFile        _SfnOpenFile
#define _DosDeleteFile      _SfnDeleteFile
#define _DosGetFileMode     _SfnGetFileMode
#define _DosChgFileMode     _SfnChgFileMode
#define _DosGetCurrentDir   _SfnGetCurrentDir
#define _DosRenameFile      _SfnRenameFile

#endif


#pragma save, call(inline=>on, reg_param=>(bx), reg_saved=>(bx,cx,dx,si,di,ds,es,st1,st2))
static int _DosCloseFile(int Hnd) =
{   MOVrb(AH,0x3E),
    INT(0x21),
    JC(0x02),
    XOR(AX,AX)
};
#pragma restore


#pragma save, call(inline=>on, reg_param=>(bx,cx,dx,ds,di,es), reg_saved=>(bx,cx,dx,si,di,ds,es,st1,st2))
static int _DosReadFile(int Hnd, unsigned Len, void *Buf, unsigned *rdLen) =
{   MOVrb(AH,0x3F),
    INT(0x21),
 #if _fdata
    JC(0x05),
    USE(ES), MOVmr(DI,AX),
 #else
    JC(0x04),
    MOVmr(DI,AX),
 #endif
    XOR(AX,AX)
};
#pragma restore


#pragma save, call(inline=>on, reg_param=>(bx,cx,dx,ds,di,es), reg_saved=>(bx,cx,dx,si,di,ds,es,st1,st2))
static int _DosWriteFile(int Hnd, unsigned Len, void *Buf, unsigned *wrLen) =
{   MOVrb(AH,0x40),
    INT(0x21),
 #if _fdata
    JC(0x05),
    USE(ES), MOVmr(DI,AX),
 #else
    JC(0x04),
    MOVmr(DI,AX),
 #endif
    XOR(AX,AX)
};
#pragma restore



#pragma save, call(inline=>on, reg_param=>(ax,bx,dx,cx), reg_saved=>(bx,cx,si,di,ds,es,st1,st2))
static int _DosSeekFile(byte Origin, int Hnd, dword Ofs) =
{   MOVrb(AH,0x42),
    INT(0x21),
    JC(0x02),
    XOR(AX,AX)
};
#pragma restore


#pragma save, call(inline=>on, reg_param=>(ax,bx,dx,cx,di,es), reg_saved=>(bx,cx,si,di,ds,es,st1,st2))
static int _DosRepositionFile(byte Origin, int Hnd, dword Ofs, dword *pOfs) =
{   MOVrb(AH,0x42),
    INT(0x21),
 #if _fdata
    JC(0x09),
    USE(ES), MOVmr(DI,AX),
    USE(ES), MOVmdr(DI,2,DX),
 #else
    JC(0x07),
    MOVmr(DI,AX),
    MOVmdr(DI,2,DX),
 #endif
    XOR(AX,AX)
};
#pragma restore


#pragma save, call(inline=>on, reg_param=>(bx,di,es), reg_saved=>(bx,si,di,ds,es,st1,st2))
static int _DosGetFilePos(int Hnd, dword *pOfs) =
{   MOVrw(AX,0x4200|_DOS_SEEKCUR),
    XOR(CX,CX),
    XOR(DX,DX),
    INT(0x21),
 #if _fdata
    JC(0x09),
    USE(ES), MOVmr(DI,AX),
    USE(ES), MOVmdr(DI,2,DX),
 #else
    JC(0x07),
    MOVmr(DI,AX),
    MOVmdr(DI,2,DX),
 #endif
    XOR(AX,AX)
};
#pragma restore


#pragma save, call(inline=>on, reg_param=>(bx,di,es), reg_saved=>(bx,si,di,ds,es,st1,st2))
static int _DosGetFileLen(int Hnd, dword *pLen) =
{   MOVrw(AX,0x4200|_DOS_SEEKEND),
    XOR(CX,CX),
    XOR(DX,DX),
    INT(0x21),
 #if _fdata
    JC(0x09),
    USE(ES), MOVmr(DI,AX),
    USE(ES), MOVmdr(DI,2,DX),
 #else
    JC(0x07),
    MOVmr(DI,AX),
    MOVmdr(DI,2,DX),
 #endif
    XOR(AX,AX)
};
#pragma restore




#pragma save, call(inline=>on, reg_param=>(bx,si,ds), reg_saved=>(bx,cx,dx,si,di,ds,es,st1,st2))
static int _DosDupFile(int Hnd, int *pHnd) =
{   MOVrb(AH,0x45),
    INT(0x21),
    JC(0x04),
    MOVmr(SI,AX),
    XOR(AX,AX)
};
#pragma restore


#pragma save, call(inline=>on, reg_param=>(bx,cx), reg_saved=>(bx,cx,dx,si,di,ds,es,st1,st2))
static int _DosForceDupFile(int Hnd, int NewHnd) =
{   MOVrb(AH,0x46),
    INT(0x21),
    JC(0x02),
    XOR(AX,AX)
};
#pragma restore




#pragma save, call(inline=>on, reg_param=>(bx, si,ds), reg_saved=>(cx,dx,si,di,ds,es,st1,st2))
static int _DosAllocSeg(unsigned Paras, unsigned * far *pSeg) =
{   MOVrb(AH,0x48),
    INT(0x21),
    JC(0x04),
    MOVrm(SI,AX),
    XOR(AX,AX)
};
#pragma restore


#pragma save, call(inline=>on, reg_param=>(), reg_return=>(bx), reg_saved=>(cx,dx,si,di,ds,es,st1,st2))
static unsigned _DosMemAvail() =
{   MOVrb(AH,0x48),
    MOVrw(BX,0xFFFF),
    INT(0x21)
};
#pragma restore


#pragma save, call(inline=>on, reg_param=>(es), reg_saved=>(bx,cx,dx,si,di,ds,es,st1,st2))
static int _DosFreeSeg(unsigned Seg) =
{   MOVrb(AH,0x49),
    INT(0x21),
    JC(0x02),
    XOR(AX,AX)
};
#pragma restore


#pragma save, call(inline=>on, reg_param=>(bx, es, si,ds), reg_saved=>(cx,dx,si,di,ds,es,st1,st2))
static int _DosReallocSeg(unsigned Paras, unsigned Seg, unsigned *pMaxParas) =
{   MOVrb(AH,0x4A),
    INT(0x21),
    JC(0x04),
    MOVrm(SI,BX),
    XOR(AX,AX)
};
#pragma restore


#pragma save, call(inline=>on, reg_param=>(ax), reg_saved=>(ax,bx,cx,dx,si,di,ds,es,st1,st2))
static void _DosProcessTerminate(unsigned Ret) =
{   MOVrb(AH,0x4C),
    INT(0x21)
};
#pragma restore


#pragma save, call(inline=>on, reg_param=>(), reg_saved=>(bx,cx,dx,si,di,ds,es,st1,st2))
static int _DosProcessReturnCode() =
{   MOVrb(AH,0x4D),
    INT(0x21)
};
#pragma restore


#pragma save, call(inline=>on, reg_param=>(), reg_saved=>(bx,cx,dx,si,di,ds,es,st1,st2))
static int _DosGetVerify() =
{   MOVrb(AH,0x54),
    INT(0x21),
    XOR(AH,AH)
};
#pragma restore


#pragma save, call(inline=>on, reg_param=>(bx,si,ds), reg_saved=>(bx,si,di,ds,es,st1,st2))
static int _DosGetFileTime(int Hnd, dword *pTime) =
{   MOVrw(AX,0x5700),
    INT(0x21),
    JC(0x07),
    MOVmr(SI,CX),
    MOVmdr(SI,2,DX),
    XOR(AX,AX)
};
#pragma restore


#pragma save, call(inline=>on, reg_param=>(bx,dx,cx), reg_saved=>(bx,cx,dx,si,di,ds,es,st1,st2))
static int _DosSetFileTime(int Hnd, dword Time) =
{   MOVrw(AX,0x5701),
    INT(0x21),
    JC(0x02),
    XOR(AX,AX)
};
#pragma restore


#pragma save, call(inline=>on, reg_param=>(), reg_saved=>(st1,st2))
static int _DosErrorCode() =
{   MOVrw(AX,0x5900),
    XOR(BX,BX),
    INT(0x21)
};
#pragma restore


#pragma save, call(inline=>on, reg_param=>(si,ds), reg_saved=>(si,ds,st1,st2))
static int _DosErrorInfo(_DOSERROR *pErr) =
{   MOVrw(AX,0x5900),
    XOR(BX,BX),
    PUSHs(DS),
    PUSH(SI),
    INT(0x21),
    POP(SI),
    POPs(DS),
 #if _fdata
    MOVrs(DX,DS),
    OR(DX,SI),
 #else
    CMPw(SI,0),
 #endif
    JE(0x08),
    MOVmr(SI,AX),
    MOVmdr(SI,2,BX),
    MOVmdr(SI,4,CH)
};
#pragma restore


#pragma save, call(inline=>on, reg_param=>(bx,dx,cx,di,si), reg_saved=>(bx,cx,dx,si,di,ds,es,st1,st2))
static int _DosLockFile(int Hnd, dword Ofs, dword Len) =
{   MOVrw(AX,0x5C00),
    INT(0x21),
    JC(0x02),
    XOR(AX,AX)
};
#pragma restore


#pragma save, call(inline=>on, reg_param=>(dx,ds,di,es,cx), reg_saved=>(bx,dx,si,di,ds,es,st1,st2))
static int _DosCreateTempFile(const char *Path, int *pHnd, byte Attr) =
{   MOVrb(AH,0x5A),
    MOVrb(CH,0),
    INT(0x21),
 #if _fdata
    JC(0x05),
    USE(ES), MOVmr(DI,AX),
 #else
    JC(0x04),
    MOVmr(DI,AX),
 #endif
    XOR(AX,AX)
};
#pragma restore


#pragma save, call(inline=>on, reg_param=>(dx,ds,di,es,cx), reg_saved=>(bx,dx,si,di,ds,es,st1,st2))
static int _DosCreateNewFile(const char *Path, int *pHnd, byte Attr) =
{   MOVrb(AH,0x5A),
    MOVrb(CH,0),
    INT(0x21),
 #if _fdata
    JC(0x05),
    USE(ES), MOVmr(DI,AX),
 #else
    JC(0x04),
    MOVmr(DI,AX),
 #endif
    XOR(AX,AX)
};
#pragma restore


#pragma save, call(inline=>on, reg_param=>(bx,dx,cx,di,si), reg_saved=>(bx,cx,dx,si,di,ds,es,st1,st2))
static int _DosUnlockFile(int Hnd, dword Ofs, dword Len) =
{   MOVrw(AX,0x5C01),
    INT(0x21),
    JC(0x02),
    XOR(AX,AX)
};
#pragma restore


#pragma save, call(inline=>on, reg_param=>(si,ds,di,es), reg_saved=>(bx,cx,dx,si,di,ds,es,st1,st2))
static int _DosTruePath(const char *Spec, char far *Path) =
{   MOVrb(AH,0x60),
#if !defined(_fdata)
    PUSHs(DS),
    POPs(ES),
#endif
    INT(0x21),
    JC(0x02),
    XOR(AX,AX)
};
#pragma restore


#pragma save, call(inline=>on, reg_param=>(bx), reg_saved=>(bx,cx,dx,si,di,ds,es,st1,st2))
static unsigned _DosSetHandleCount(int Cnt) =
{   MOVrb(AH,0x67),
    INT(0x21),
    JC(0x02),
    XOR(AX,AX)
};
#pragma restore


#pragma save, call(inline=>on, reg_param=>(bx), reg_saved=>(bx,cx,dx,si,di,ds,es,st1,st2))
static unsigned _DosCommitFile(int Hnd) =
{   MOVrb(AH,0x68),
    INT(0x21),
    JC(0x02),
    XOR(AX,AX)
};
#pragma restore

#pragma save, call(inline=>on, reg_param=>(bx), reg_saved=>(bx,dx,si,di,ds,es,st1,st2))
static unsigned _DosFileTruncate(int Hnd) =
{
    XOR(CX,CX),
    MOVrb(AH,0x40),
    INT(0x21),
    JC(0x02),
    XOR(AX,AX)
};
#pragma restore

#pragma save, call(inline=>on, reg_param=>(bx,di,es), reg_saved=>(bx,si,di,ds,es,st1,st2))
static unsigned _DosFileDate(int Hnd,dword *pdatetime) =
{
    MOVrw(AX,0x5700),
    INT(0x21),
 #if _fdata
    JC(0x09),
    USE(ES), MOVmr(DI,CX),
    USE(ES), MOVmdr(DI,2,DX),
 #else
    JC(0x07),
    MOVmr(DI,CX),
    MOVmdr(DI,2,DX),
 #endif
    XOR(AX,AX)
};
#pragma restore

#pragma save, call(inline=>on, reg_param=>(bx,cx,dx), reg_saved=>(bx,si,di,ds,es,st1,st2))
static unsigned _DosSetFileDate(int Hnd,dword datetime) =
{
    MOVrw(AX,0x5701),
    INT(0x21),
    JC(0x02),
    XOR(AX,AX)
};
#pragma restore

#endif /* _DOS_H */
